import * as Types from './json-types';
export declare type PromiseOrValue<T> = T | PromiseLike<T>;
export interface PromiseOrValueModule {
    /** Apply a transformation to the result of a promise or value */
    map: <T, U>(promiseOrValue: PromiseOrValue<T>, onFulfilled: (value: T) => PromiseOrValue<U>) => PromiseOrValue<U>;
    /** Attach callbacks for the resolution/rejection of the result of the action */
    then: <T, U>(action: () => PromiseOrValue<T>, onFulfilled: (value: T) => PromiseOrValue<U>, onRejected?: (reason: any) => PromiseOrValue<U>) => PromiseOrValue<U>;
    /** Handle errors thrown by the action */
    catch: <T>(action: () => PromiseOrValue<T>, onRejected: (reason: any) => T) => PromiseOrValue<T>;
    /** Turn a promise or value into a promise */
    asPromise: <T>(promiseOrValue: PromiseOrValue<T>) => PromiseLike<T>;
}
/** Functions for when a result may be either synchronous (a value) or asynchronous (a `Promise`) */
export declare const PromiseOrValue: PromiseOrValueModule;
/**
 * An Alexa `Handler` takes a raw request and return a raw response either synchronously or asynchronously.
 *
 * _Only in the special case of the `SessionEnded` request, the response should be void._
 */
export declare type Handler = (event: Types.RequestBody) => PromiseOrValue<Types.ResponseBody | void>;
/**
 * An Alexa Middleware `Pipe` is the same as the Alexa Handler with the addition of having the option of
 * calling the next step in the pipe.
 */
export declare type Pipe = (event: Types.RequestBody, next: Handler) => PromiseOrValue<Types.ResponseBody | void>;
/**
 * The words to be spoken.
 *
 * _If `SSML` is specified, the `Text` will be ignored._
 */
export interface Speech {
    Text?: string;
    SSML?: string;
}
export interface LinkAccount {
    Type: 'LinkAccount';
}
export interface CardImage {
    /** 720w x 480h */
    SmallUrl: string;
    /** 1200w x 800h */
    LargeUrl: string;
}
/**
 * Short-hand model for cards. _The `Type` can either be `LinkAccount` or `Card`._
 *
 * _This infers if the card is "Simple" or "Standard" based on the use of the `Image` property._
 */
export declare type Card = LinkAccount | {
    Type: 'Card';
    /** Title of the card */
    Title: string;
    /**
     * A string containing the text content.
     *
     * _New lines can be added with either `\n` or `\r\n`._
     */
    Content: string;
    /** Optional image to be displayed */
    Image?: CardImage;
};
/** Short-hand response from an intent or launch request */
export interface Response<State> {
    Say?: Speech;
    NewState?: State;
    Reprompt?: Speech;
    Card?: Card;
    EndSession?: boolean;
}
export declare type HandlerResult<State> = PromiseOrValue<Response<State> | Types.ResponseBody | void>;
/**
 * Mapping of the `name` of the slot to the `value`.
 *
 * _This uses the `name` property, not the slot key._
 */
export declare type Slots = Map<string, any>;
export declare type IntentHandler<State> = (sessionState: State, slots: Slots, request: Types.RequestBody, next: Handler) => HandlerResult<State>;
export interface StandardIntentRoutes<State> {
    /** Built-in AMAZON.CancelIntent. */
    Cancel?: IntentHandler<State>;
    /** Built-in AMAZON.HelpIntent. */
    Help?: IntentHandler<State>;
    /** Built-in AMAZON.LoopOffIntent. */
    LoopOff?: IntentHandler<State>;
    /** Built-in AMAZON.LoopOnIntent. */
    LoopOn?: IntentHandler<State>;
    /** Built-in AMAZON.NextIntent. */
    Next?: IntentHandler<State>;
    /** Built-in AMAZON.NoIntent. */
    No?: IntentHandler<State>;
    /** Built-in AMAZON.PauseIntent. */
    Pause?: IntentHandler<State>;
    /** Built-in AMAZON.PreviousIntent. */
    Previous?: IntentHandler<State>;
    /** Built-in AMAZON.RepeatIntent. */
    Repeat?: IntentHandler<State>;
    /** Built-in AMAZON.ResumeIntent. */
    Resume?: IntentHandler<State>;
    /** Built-in AMAZON.ShuffleOffIntent. */
    ShuffleOff?: IntentHandler<State>;
    /** Built-in AMAZON.ShuffleOnIntent. */
    ShuffleOn?: IntentHandler<State>;
    /** Built-in AMAZON.StartOverIntent. */
    StartOver?: IntentHandler<State>;
    /** Built-in AMAZON.StopIntent. */
    Stop?: IntentHandler<State>;
    /** Built-in AMAZON.YesIntent. */
    Yes?: IntentHandler<State>;
}
export interface Routes<State> {
    /** Default state when no state has yet been specified. */
    InitialState: State;
    /** Executed when the skill is started without an intent. */
    Launch?: IntentHandler<State>;
    /** Executed after a session was ended (e.g. by the user saying "stop" or not responding. */
    SessionEnded?: () => PromiseOrValue<void>;
    /** Standard built-in intents. */
    Standard?: StandardIntentRoutes<State>;
    /** Custom named intent handlers. */
    Custom?: Iterable<[string, IntentHandler<State>]>;
}
export interface StateModule {
    /** The string key used to store state from AlexaTs in the Alexa session attributes */
    attributeKey: string;
    /** Get the state (if available) from the request, otherwise return the initial state. */
    fromRequest: <State>(request: Types.RequestBody, initialState?: State) => State;
}
/** Functions for accessing raw AlexaTs state */
export declare const State: StateModule;
/** Expand a short-hand `Response` into a raw Alexa `ResponseBody` */
export declare const response: <State>(response: Response<State>, previousState?: State | undefined) => Types.ResponseBody;
/** Functions for creating and composing middleware `Pipe`s */
export interface PipeModule {
    /** Create a pipe by calling a series of child pipes. */
    join: (steps: Pipe[]) => Pipe;
    /** Handle specific request types and intents. */
    router: <State>(routes: Routes<State>) => Pipe;
    /** Catch and handle errors from subsequent handlers. */
    catch: (onError: ((error: any) => any)) => Pipe;
    /** Log the incoming request and the response or error thrown by the subsequent handlers. */
    tracer: (logger?: (message: string, obj: any) => void) => Pipe;
    /** Just call the next step in the pipe. */
    doNothing: () => Pipe;
    /** Convert a pipe into a handler. */
    toHandler: (pipe: Pipe) => Handler;
}
export declare const Pipe: PipeModule;
/** Functions for creating an Alexa handler */
export interface HandlerModule {
    /** Create a handler from a middleware pipeline */
    middleware: (pipe: Pipe) => Handler;
    /** Create a handler from request & intent routes */
    router: <State>(routes: Routes<State>) => Handler;
    /** Create a handler from middleware pipeline steps */
    pipe: (steps: Pipe[]) => Handler;
}
export declare const Handler: HandlerModule;
/** Functions for creating an AWS lambda handler */
export interface LambdaModule {
    /** Create a lambda handler from an Alexa Handler */
    handler: (handler: Handler) => Types.AlexaLambda;
    /** Create a lambda handler from request & intent routes */
    router: <State>(routes: Routes<State>) => Types.AlexaLambda;
    /** Create a lambda handler from a middleware pipeline */
    pipe: (steps: Pipe[]) => Types.AlexaLambda;
}
/** Functions for creating an AWS lambda handler */
export declare const Lambda: LambdaModule;
