"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Helpers to create various fake things for Alexa requests.
 */
exports.Fake = Object.freeze({
    timestamp: () => new Date().toISOString(),
    userId: () => 'amzn1.ask.account.FAKEACCOUNTID',
    requestId: () => 'amzn1.echo-api.request.d15314a4-3a86-4d5a-90b0-5adb6a5b5ce7',
    sessionId: () => 'amzn1.echo-api.session.bcea274f-0dd8-43ac-8369-e102f881b42e',
    applicationId: () => 'amzn1.ask.skill.659a0b41-d887-45b0-a158-01b3d2214d5a',
    /**
     * Create a version 1.0 Alexa request
     */
    requestBody: (session, request, context) => ({
        version: '1.0',
        session: session,
        request: request,
        context: context,
    }),
    launchRequest: () => ({
        type: 'LaunchRequest',
        requestId: exports.Fake.requestId(),
        timeStamp: exports.Fake.timestamp(),
    }),
    intentRequest: (intent, slots) => ({
        type: 'IntentRequest',
        requestId: exports.Fake.requestId(),
        timeStamp: exports.Fake.timestamp(),
        intent: {
            name: intent,
            slots: slots || {},
        },
    }),
    sessionEnded: (reason) => ({
        type: 'SessionEndedRequest',
        reason: reason || 'USER_INITIATED',
        requestId: exports.Fake.requestId(),
        timeStamp: exports.Fake.timestamp(),
    }),
    session: () => ({
        sessionId: exports.Fake.sessionId(),
        application: {
            applicationId: exports.Fake.applicationId(),
        },
        attributes: {},
        user: {
            userId: exports.Fake.userId(),
        },
        new: true,
    }),
    requestContext: () => ({
        AudioPlayer: {
            playerActivity: 'IDLE',
        },
        System: {
            apiEndpoint: 'https://api.amazonalexa.com',
            application: {
                applicationId: exports.Fake.applicationId(),
            },
            device: {
                deviceId: 'string',
                supportedInterfaces: {
                    AudioPlayer: {},
                },
            },
            user: {
                userId: exports.Fake.userId(),
                accessToken: undefined,
            },
        },
    }),
    lambdaContext: () => ({
        awsRequestId: 'e0aa9f2d-5f4a-4ae8-a7b9-bb1a79ed91f6',
        functionName: 'test-function',
        callbackWaitsForEmptyEventLoop: true,
        functionVersion: '$LATEST',
        invokeid: 'invokeid',
        logGroupName: 'logGroupName',
        logStreamName: 'logStreamName',
        memoryLimitInMB: '128',
    }),
});
exports.configure = (handler, lambdaContext) => {
    const lambdaContextOrDefault = typeof lambdaContext === 'undefined' ? exports.Fake.lambdaContext() : lambdaContext;
    return request => new Promise((resolve, reject) => handler(request, lambdaContextOrDefault, (err, res) => {
        if (err) {
            reject(err);
        }
        else {
            resolve(res);
        }
    }));
};
const defaultBuilders = () => ({
    LaunchRequest: exports.Fake.launchRequest,
    IntentRequest: exports.Fake.intentRequest,
    SessionEndedRequest: exports.Fake.sessionEnded,
    Session: exports.Fake.session,
    Context: exports.Fake.requestContext,
});
const extendBuilders = (custom) => {
    const builders = defaultBuilders();
    if (typeof custom === 'undefined') {
        return builders;
    }
    Object.keys(custom).forEach(key => {
        builders[key] = custom[key];
    });
    return builders;
};
/**
 * Mock session for an Alexa lambda
 */
class Session {
    /**
     * Start a new session
     * @param lambda Alexa lambda function to test.
     * @param builders Optional custom builders to be used.
     */
    constructor(lambda, builders) {
        this.execute = exports.configure(lambda);
        this.builders = extendBuilders(builders);
        this.session = this.builders.Session();
        this.context = this.builders.Context();
        this.context.System.user = this.session.user;
    }
    LinkAccount(accessToken) {
        this.session.user.accessToken = accessToken;
        return this;
    }
    LaunchSkill() {
        return this.Request(exports.Fake.requestBody(this.session, this.builders.LaunchRequest(), this.context));
    }
    RequestIntent(name, slots) {
        return this.Request(exports.Fake.requestBody(this.session, this.builders.IntentRequest(name, slots), this.context));
    }
    EndSession(reason) {
        return this.Request(exports.Fake.requestBody(this.session, this.builders.SessionEndedRequest(reason), this.context));
    }
    Request(request) {
        return this.execute(request).then(response => {
            this.session.new = false;
            if (response &&
                response.sessionAttributes &&
                Object.getOwnPropertyNames(response.sessionAttributes).length > 0) {
                this.session.attributes = response.sessionAttributes;
            }
            return response;
        });
    }
}
exports.Session = Session;
//# sourceMappingURL=testing.js.map